/***************************************************************************
* Copyright (C) 2009 The Terminal Prototype
* All rights reserved.
* Contact: florian (florian.rueberg@rwth-aachen.de)
*
* Terminal Prototype is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Terminal Prototype is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Terminal Prototype.  If not, see <http://www.gnu.org/licenses/>.
******************************************************************************/

#include "dialog.h"

#include <QtGui/QFileDialog>
#include <QtGui/QDialog>
#include <QtGui/QFrame>
#include <QtGui/QMessageBox>
#include <QtGui>
#include <iostream>
#include <fstream>
#include <string>

#include "myKeyBoard.h"
#include "textBrowser.h"
#include "ui/line_edit.h"
#include "ui/buttons.h"
#include "ui/text_browser.h"
#include "ui/keyboard.h"
#include "ui/message_dialog.h"
#include "sys/sys.h"
#include "lineEdit.h"

class TabBar : public QTabBar{
public:

QWidget *parent;


TabBar(QWidget *parent = 0)
	: QTabBar() {
	this->parent = parent;
}

bool event(QEvent* event){
    bool ret = QTabBar::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);
        return true;
    }
    return ret;
}

void keyPressEvent(QKeyEvent* event){	
	QTabBar::keyPressEvent(event);
	if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(parent, onyx::screen::ScreenProxy::GC);
	}else if (event->key() == Qt::Key_Escape){
		QApplication::exit();
	}
}

void mousePressEvent(QMouseEvent *event){
	QTabBar::mousePressEvent(event);
	onyx::screen::ScreenProxy::instance().flush(parent, onyx::screen::ScreenProxy::GC);
}

};

class TabWidget : public QTabWidget{
public:

TabWidget(QWidget *parent = 0)
	: QTabWidget(parent) {
	TabBar *b = new TabBar(parent);
	setTabBar(b);
}

bool event(QEvent* event){
    bool ret = QTabWidget::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
        return true;
    }
    return ret;
}

void keyPressEvent(QKeyEvent* event){	
	QTabWidget::keyPressEvent(event);
	if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);
	}else if (event->key() == Qt::Key_Escape){
		QApplication::exit();
	}
	
}

};

class FileDialog : public QFileDialog{
public:

FileDialog(QWidget *parent = 0, const QString & caption = QString(), const QString & directory = QString(), const QString & filter = QString())
	: QFileDialog(parent,caption,directory,filter) {
}

bool event(QEvent* event){
    bool ret = QFileDialog::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
        return true;
    }
    return ret;
}

void keyPressEvent(QKeyEvent* event){	
	QFileDialog::keyPressEvent(event);
	if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);
	}
}



};

class Frame : public QFrame{
public:

QWidget *parent;

Frame(QWidget *parent = 0, Qt::WindowFlags f = 0)
	: QFrame(parent,f) {
	this->parent = parent;
}

void focusOutEvent( QFocusEvent *f ){
	setVisible(false);
}

bool event(QEvent* event){
    bool ret = QFrame::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(parent, onyx::screen::ScreenProxy::GU);
        return true;
    }
    return ret;
}

void keyPressEvent(QKeyEvent* event){	
	if (event->key() == Qt::Key_Escape){
		QApplication::exit();
	}else if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);
	}
}

};

class Action : public ui::OnyxPushButton{
public:

QWidget *parent;

Action(QIcon i, QString s, QWidget *parent = 0)
	: ui::OnyxPushButton(i, s, parent) {
	this->parent = parent;
	setStyleSheet(NORMAL_STYLE);
}

Action(QString s, QWidget *parent = 0)
	: ui::OnyxPushButton(s, parent) {
	this->parent = parent;
	setStyleSheet(NORMAL_STYLE);
}

void keyPressEvent(QKeyEvent* event){	
	if (event->key() == Qt::Key_Escape){
		QApplication::exit();
	}else if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);
	}
};

bool event(QEvent* event){
    bool ret = ui::OnyxPushButton::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(parent, onyx::screen::ScreenProxy::GU);
        return true;
    }
    return ret;
}

void mousePressEvent(QMouseEvent *event){
	emit(pressed());
}

};

class Menu : public ui::OnyxPushButton{
public:

Dialog *parent;
QWidget *directParent;
Frame *frame;
QVBoxLayout *layout;

Menu(QString s, Dialog *parent = 0, QWidget *directParent = 0)
	: ui::OnyxPushButton(s,parent) {
	this->parent = parent;
	this->directParent = directParent;
	setFixedWidth(100);
	setFixedHeight(40);
	setStyleSheet(NORMAL_STYLE);

	frame = new Frame(parent,Qt::Popup);
	layout = new QVBoxLayout;
	layout->setSpacing(3);
	layout->setMargin(1);
	frame->setLayout(layout);
	frame->setStyleSheet(THIN_STYLE);
}

void mousePressEvent(QMouseEvent *event){
	parent->frame = frame;
	frame->setFocus();
	frame->move(x() + directParent->x(),y() + directParent->y()+directParent->height()+1);
	frame->setVisible(true);
	onyx::screen::ScreenProxy::instance().flush(frame, onyx::screen::ScreenProxy::GU);
};

void addAction(Action *a){
	layout->addWidget(a);
	a->setFixedHeight(40);
}

};

class MenuBar : public QWidget{
public:

QWidget *parent;
QHBoxLayout *layout;

MenuBar( QWidget *parent = 0)
	: QWidget(parent) {
	this->parent = parent;
	layout = new QHBoxLayout;
	layout->setSpacing(3);
	layout->setMargin(0);
	setLayout(layout);
}

void addMenu(Menu *menu ){
	layout->addWidget(menu);
}

void addStretch(){
	layout->addStretch(100);
}

};






TextBrowser::TextBrowser(QWidget *parent)
	: QTextBrowser(parent) {
	setReadOnly(false);
}

void TextBrowser::keyPressEvent(QKeyEvent* event){
	if (event->key() != Qt::Key_Escape)	
		QTextBrowser::keyPressEvent(event);
	if (event->key() == Qt::Key_Escape){
		((Dialog *)parent())->exit();
	}else if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(((QWidget *)parent()), onyx::screen::ScreenProxy::GC);
	}else if (event->key() == Qt::Key_Up){
		QTextCursor c = this->textCursor();
		c.movePosition(QTextCursor::Up,QTextCursor::MoveAnchor);
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_Left){
		QTextCursor c = this->textCursor();
		c.movePosition(QTextCursor::Left,QTextCursor::MoveAnchor);
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_Right){
		QTextCursor c = this->textCursor();
		c.movePosition(QTextCursor::Right,QTextCursor::MoveAnchor);
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_Down){
		QTextCursor c = this->textCursor();
		c.movePosition(QTextCursor::Down,QTextCursor::MoveAnchor);
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_Return){
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_PageDown){
		redo();
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_PageUp){
		undo();
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}
};

void TextBrowser::mouseReleaseEvent(QMouseEvent *event){
	QTextBrowser::mouseReleaseEvent(event);
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
};

bool TextBrowser::event(QEvent* event){
    bool ret = QTextBrowser::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
        return true;
    }
    return ret;
}


Dialog::Dialog(QWidget *parent)
: QDialog(parent)
{
	
	CanUndo=false;
	SaveMode=false;
	CurrentFile = "";
	CurrentDirectory = "/media/sd/";

	MenuBar *menuBar = new MenuBar(this);
	Menu *file = new Menu(tr("&Файл"),this,menuBar);
	Menu *edit = new Menu(tr("&Редактировать"),this,menuBar);
	edit->setMinimumWidth(220);	
	menuBar->addMenu(file);
	menuBar->addMenu(edit);
	menuBar->addStretch();

	QIcon iopen("/media/sd/images/open.png");
	a1 = new Action(iopen, tr("&Открыть"), this);
	file->addAction(a1);
	a1->setFixedWidth(220);
	connect(a1,SIGNAL(pressed()),this,SLOT(open()));
	
	QIcon isave("/media/sd/images/save.png");
	a4 = new Action(isave, tr("&Сохранить"), this);
	file->addAction(a4);
	a4->setFixedWidth(220);
	a4->setEnabled(false);
	
	connect(a4,SIGNAL(pressed()),this,SLOT(SimpleSave()));

	a2 = new Action(isave, tr("&Сохранить как"), this);
	file->addAction(a2);
	a2->setFixedWidth(220);

	connect(a2,SIGNAL(pressed()),this,SLOT(save()));

	Action *a3 = new Action(tr("&Выход"), this);
	file->addAction(a3);
	a3->setFixedWidth(220);

	connect(a3,SIGNAL(pressed()),this,SLOT(exit()));


	QIcon icopy("/media/sd/images/copy.png");
	Action *acopy = new Action(icopy, tr("&Копировать"), this);
	edit->addAction(acopy);
	acopy->setFixedWidth(200);
	connect(acopy,SIGNAL(pressed()),this,SLOT(copy()));

	QIcon icut("/media/sd/images/cut.png");
	Action *acut = new Action(icut, tr("&Вырезать"), this);
	edit->addAction(acut);
	acut->setFixedWidth(200);
	connect(acut,SIGNAL(pressed()),this,SLOT(cut()));

	QIcon ipaste("/media/sd/images/paste.png");
	Action *apaste = new Action(ipaste, tr("&Вставить"), this);
	edit->addAction(apaste);
	apaste->setFixedWidth(200);
	connect(apaste,SIGNAL(pressed()),this,SLOT(paste()));
	

	QHBoxLayout *SaveLayout = new QHBoxLayout;
	Filename = new LineEdit;
	Filename->setText("");
	Filename->setEnabled(false);
	sOK = new ui::OnyxPushButton(tr("ОК"),0);
	sOK->setEnabled(false);
	sCancel = new ui::OnyxPushButton(tr("Отмена"),0);
	sCancel->setEnabled(false);
	connect(sOK,SIGNAL(clicked()),this,SLOT(SaveOkClicked()));
	connect(sCancel,SIGNAL(clicked()),this,SLOT(SaveCancelClicked()));
	QLabel *sLbl= new QLabel(tr("Имя файла: "));

	QVBoxLayout *layout = new QVBoxLayout;
	QLabel *info = new QLabel(tr("<b>EditFlo v0.2 Руссификация - Ilyich build 20100609</b>"), this);
	text = new TextBrowser(this);
	text->setUndoRedoEnabled(true);
	connect(text,SIGNAL(undoAvailable(bool)),this,SLOT(UndoStatus(bool)));
	TabWidget *tabw = new TabWidget(this);
	keyboard = new MyKeyBoard( this, text,0,Filename);
	MyKeyBoard *sonder_keyboard = new MyKeyBoard( this, text, 1,Filename);
        MyKeyBoard *russian_keyboard = new MyKeyBoard( this, text, 2,Filename);
	connect(text->horizontalScrollBar(),SIGNAL(valueChanged(int)),this,SLOT(SlidersMove(int)));
        connect(text->verticalScrollBar(),SIGNAL(valueChanged(int)),this,SLOT(SlidersMove(int)));	

	info->setAlignment(Qt::AlignRight);
	text->setStyleSheet(TEXTBROWSER_STYLE);
	menuBar->setStyleSheet(NORMAL_STYLE);
	tabw->setStyleSheet(TAB_STYLE);
	info->setFocus();

	

	SaveLayout->addWidget(sLbl);
	SaveLayout->addWidget(Filename);
	SaveLayout->addWidget(sOK);
	SaveLayout->addWidget(sCancel);
	SaveLayout->addStretch();
	
	layout->addWidget(info);
	layout->addWidget(menuBar);	
	layout->addWidget(text);
	layout->addLayout(SaveLayout);
	tabw->addTab(keyboard, tr("Английский"));
	tabw->addTab(sonder_keyboard, tr("Символы"));   
	tabw->addTab(russian_keyboard, tr("Русский"));	
  	layout->addWidget(tabw);

    	this->setLayout(layout);
}

Dialog::~Dialog(){
}

void Dialog::exit(){
	if (getCanUndo()){		
		ui::MessageDialog *msg = new ui::MessageDialog(QMessageBox::Question,tr("Выход"),tr("Вы действительно хотите выйти, не сохранив изменения?"),QMessageBox::Yes | QMessageBox::No,0);			
		if (msg->exec() == QMessageBox::No){
			onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
			delete msg;		
		}
		else {
			delete msg;
			QApplication::exit();
		} 
		
	} else
		QApplication::exit();
}

void Dialog::save(){
	/*frame->hide();
	
	FileDialog *fdlg = new FileDialog(this,tr("Сохранить файл"),"/media/sd",tr("Текстовый файл (*.txt)"));

	QFile file(curFile);
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
        	return;
	}

	QTextStream out(&file);
	out << text->toPlainText();
	text->setPlainText(text->toPlainText());
	CanUndo=false;
	onyx::screen::ScreenProxy::instance().flush(text, onyx::screen::ScreenProxy::GU);*/
	frame->hide();	
	SaveMode=true;
	text->setEnabled(false);
	Filename->setEnabled(true);
	sOK->setEnabled(true);
	sCancel->setEnabled(true);
	a1->setEnabled(false);
	a2->setEnabled(false);
	a4->setEnabled(false);
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
}

void Dialog::open(){
	frame->hide();
	QString name = Dialog::getFileName();
	QFile file(name);
    	if (!file.open(QFile::ReadOnly | QFile::Text)) return;

    	QTextStream in(&file);
	text->setPlainText(in.readAll());
	CanUndo=false;
	int SlPos = name.lastIndexOf("/");
	CurrentFile=name.right(name.length()-1-SlPos);
	CurrentDirectory=name.left(SlPos+1);
	Filename->setText(CurrentFile);
	a4->setEnabled(true);
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
}

void Dialog::copy(){
	frame->hide();
	if (!SaveMode) text->copy(); else Filename->copy();
	onyx::screen::ScreenProxy::instance().flush(text, onyx::screen::ScreenProxy::GU);
}

void Dialog::cut(){
	frame->hide();
	if (!SaveMode) text->cut(); else Filename->cut();
	onyx::screen::ScreenProxy::instance().flush(text, onyx::screen::ScreenProxy::GU);
}

void Dialog::paste(){
	frame->hide();
	if (!SaveMode) text->paste(); else Filename->paste();
	onyx::screen::ScreenProxy::instance().flush(text, onyx::screen::ScreenProxy::GU);
}

void Dialog::UndoStatus(bool res){
	CanUndo=res;
}

bool Dialog::getCanUndo(){
	return CanUndo;
}

bool Dialog::getSaveMode(){
	return SaveMode;
}

void Dialog::SaveCancelClicked(){
	frame->hide();	
	SaveMode=false;
	text->setEnabled(true);
	Filename->setEnabled(false);
	sOK->setEnabled(false);
	sCancel->setEnabled(false);
	a1->setEnabled(true);
	a2->setEnabled(true);
	if (CurrentFile.length()>0)	
		a4->setEnabled(true);
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
}

void Dialog::SaveOkClicked(){
	frame->hide();
	
	SaveMode=false;
	text->setEnabled(true);
	Filename->setEnabled(false);
	sOK->setEnabled(false);
	sCancel->setEnabled(false);
	a1->setEnabled(true);
	a2->setEnabled(true);
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);

	if (Filename->text() == "") return ;  
	FileDialog *f = new FileDialog(this,tr("Папка сохранения"),CurrentDirectory,tr("Папка"));
	f->setFileMode(QFileDialog::Directory);
	f->setViewMode(QFileDialog::Detail);
	f->setDirectory(CurrentDirectory);
	f->resize(500, 300);
	f->setAcceptMode(QFileDialog::AcceptSave);

	
	
	QStringList fileNames;
	if (f->exec()) fileNames = f->selectedFiles();
	f->close();
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);

    	if (fileNames.isEmpty()) return ;
	//fileNames.at(0)+=Filename->text();
	QString Dir=fileNames.at(0);
	if (Dir.at(Dir.length()-1)!='/')
		Dir=Dir + "/";
	QFile file(Dir+Filename->text());
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
		ui::MessageDialog *msg = new ui::MessageDialog(QMessageBox::Warning,tr("Ошибка"),tr("Не удалось сохранить файл!"),QMessageBox::Ok,0);     
		msg->exec();
		delete msg;   	
		return;
	}

	QTextStream out(&file);
	out << text->toPlainText();
	text->setPlainText(text->toPlainText());
	CanUndo=false;
	CurrentFile = Filename->text();
	CurrentDirectory = Dir;	
	a4->setEnabled(true);
}

void Dialog::SimpleSave(){
	frame->hide();	
	QFile file(CurrentDirectory+CurrentFile);
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
		ui::MessageDialog *msg = new ui::MessageDialog(QMessageBox::Warning,tr("Ошибка"),tr("Не удалось сохранить файл!"),QMessageBox::Ok,0);     
		msg->exec();
		delete msg;   	
		return;
	}

	QTextStream out(&file);
	out << text->toPlainText();
	text->setPlainText(text->toPlainText());
	CanUndo=false;
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
}

QString Dialog::getFileName(){
	FileDialog *f = new FileDialog(this,tr("Открыть файл"),CurrentDirectory,tr("Любой файл (*)")); 
	f->setFileMode(QFileDialog::ExistingFile);
	f->setViewMode(QFileDialog::Detail);
	f->setDirectory(CurrentDirectory);
	f->resize(500, 300);
	f->setAcceptMode(QFileDialog::AcceptOpen);

	QStringList fileNames;
	if (f->exec()) fileNames = f->selectedFiles();
	f->close();
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);

    	if (fileNames.isEmpty()) return tr("");

    	QString fileName = fileNames.at(0);
    	if (fileName != 0) {
		return fileName;
	}
	return tr("");
}




void LineEdit::keyPressEvent(QKeyEvent* event){
	if (event->key() != Qt::Key_Escape && event->key() !=Qt::Key_Left && event->key() != Qt::Key_Right && event->key() != Qt::Key_Return)	
		QLineEdit::keyPressEvent(event);
	if (event->key() == Qt::Key_Escape){
		((Dialog *)parent())->SaveCancelClicked();
	}else if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);
	}else if (event->key() == Qt::Key_Left){
		cursorBackward(false);
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_Right){
		cursorForward(false);
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_Return){
		((Dialog *)parent())->SaveOkClicked();
	}else if (event->key() == Qt::Key_PageDown){
		redo();
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}else if (event->key() == Qt::Key_PageUp){
		undo();
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
	}
};

void LineEdit::mouseReleaseEvent(QMouseEvent *event){
	QLineEdit::mouseReleaseEvent(event);
	onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
};

bool LineEdit::event(QEvent* event){
    bool ret = QLineEdit::event(event);
    if (event->type() == QEvent::UpdateRequest){
        onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GU);
        return true;
    }
    return ret;
}

void Dialog::SlidersMove(int value){
 onyx::screen::ScreenProxy::instance().flush(text, onyx::screen::ScreenProxy::GU);
}

