/***************************************************************************
* Copyright (C) 2009 The Terminal Prototype
* All rights reserved.
* Contact: florian (florian.rueberg@rwth-aachen.de)
*
* Terminal Prototype is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Terminal Prototype is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Terminal Prototype.  If not, see <http://www.gnu.org/licenses/>.
******************************************************************************/

#include "myKeyBoard.h"
#include "dialog.h"

#include <QtGui>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sstream>

#include "screen/screen_proxy.h"
#include "ui/keyboard.h"
#include "ui/buttons.h"
#include "ui/line_edit.h"
#include "sys/sys.h"
#include "lineEdit.h"

#include "textBrowser.h"

class Key : public ui::OnyxPushButton {
private:

MyKeyBoard *parent;

public:

char *st1;

Key( QString s, MyKeyBoard *parent, char *st1)
	: ui::OnyxPushButton(s, parent) {
	this->parent = parent;
	this->st1 = st1;
};

void keyPressEvent(QKeyEvent* event){
	if (event->key() == Qt::Key_Escape){
		QApplication::exit();
	}else if(event->key() == Qt::Key_Menu){	
		onyx::screen::ScreenProxy::instance().flush(this, onyx::screen::ScreenProxy::GC);		
	}
};

void mousePressEvent ( QMouseEvent *e){
	parent->pressed(st1);	
};

};



inline Key *add_keyboard_button(MyKeyBoard* parent, QGridLayout *grid, char* title1, char* title2, int x, int y, int width){
	Key *button;// = parent->shift ? new Key(title2,parent,title2) : new Key(title1,parent,title1);
	if (parent->shift==0 || parent->shift==2)
         button = new Key(QApplication::tr(title1),parent,title1);
	else
	 button = new Key(QApplication::tr(title2),parent,title2);	
	button->setFixedHeight(parent->key_height);
        button->setStyleSheet(NORMAL_STYLE);
	grid->addWidget(button,x,y,1,width);
	return button;
}

inline Key *add_keyboard_button_nongrid(MyKeyBoard* parent, QHBoxLayout *grid, char* title1, char* title2, char* title3, int width){
	Key *button;
        switch (parent->shift) {
         case 0: button = new Key(QApplication::tr(title1),parent,title1); break;
         case 1: button = new Key(QApplication::tr(title2),parent,title2); break;
         case 2: button = new Key(QApplication::tr(title3),parent,title3); break;
        }
	button->setFixedHeight(parent->key_height);
        button->setStyleSheet(NORMAL_STYLE);
	grid->addWidget(button,width);
	return button;
}



MyKeyBoard::MyKeyBoard(QWidget* parent, TextBrowser *t, int shift, LineEdit *fn)
    :  QWidget(parent) //ui::KeyBoard(parent) 
 {
	CAPS = "Caps";
	SPACE = "Space";
	SHIFT = "Shift";
	BACKSPACE = "Backspace";
	ENTER = "ENTER"; 
	COPY = "Copy"; 
	PASTE = "Paste"; 

	key_height = 50;
	
	cap_on = false;
	this->shift = shift;
	
	filename=fn;
	
	this->text = t;
	this->parent = parent;
	QGridLayout *grid = new QGridLayout;
	grid->setSpacing(3);
	grid->setMargin(0);

	add_keyboard_button(this, grid, "1", "!", 0, 0, 1);
	add_keyboard_button(this, grid, "2", "@", 0, 1, 1);
	add_keyboard_button(this, grid, "3", "#", 0, 2, 1);
	add_keyboard_button(this, grid, "4", "$", 0, 3, 1);
	add_keyboard_button(this, grid, "5", "%", 0, 4, 1);
	add_keyboard_button(this, grid, "6", "^", 0, 5, 1);
	add_keyboard_button(this, grid, "7", " ", 0, 6, 1);
	add_keyboard_button(this, grid, "8", "*", 0, 7, 1);
	add_keyboard_button(this, grid, "9", "(", 0, 8, 1);
	add_keyboard_button(this, grid, "0", ")", 0, 9, 1);

	QHBoxLayout *FirstRow = new QHBoxLayout;
        FirstRow->setSpacing(3);
	FirstRow->setMargin(0);
        add_keyboard_button_nongrid(this, FirstRow, "q", "~","й", 1);
	add_keyboard_button_nongrid(this, FirstRow, "w", "`","ц", 1);
	add_keyboard_button_nongrid(this, FirstRow, "e", "-","у", 1);
	add_keyboard_button_nongrid(this, FirstRow, "r", "_","к", 1);
	add_keyboard_button_nongrid(this, FirstRow, "t", "+","е", 1);
	add_keyboard_button_nongrid(this, FirstRow, "y", "=","н", 1);
	add_keyboard_button_nongrid(this, FirstRow, "u", "{","г", 1);
	add_keyboard_button_nongrid(this, FirstRow, "i", "}","ш", 1);
	add_keyboard_button_nongrid(this, FirstRow, "o", "[","щ", 1);
	add_keyboard_button_nongrid(this, FirstRow, "p", "]","з", 1);
        if (shift==2){
	 add_keyboard_button_nongrid(this, FirstRow, "", "","х", 1);
	 add_keyboard_button_nongrid(this, FirstRow, "", "","ъ", 1);	
	}
	grid->addLayout(FirstRow,1,0,1,10);

	
	FirstRow = new QHBoxLayout;
        FirstRow->setSpacing(3);
	FirstRow->setMargin(0);
	add_keyboard_button_nongrid(this, FirstRow, "a", ":","ё", 1);
	add_keyboard_button_nongrid(this, FirstRow, "s", ".","ф", 1);
	add_keyboard_button_nongrid(this, FirstRow, "d", "\"","ы", 1);
	add_keyboard_button_nongrid(this, FirstRow, "f", "'","в", 1);
	add_keyboard_button_nongrid(this, FirstRow, "g", "<","а", 1);
	add_keyboard_button_nongrid(this, FirstRow, "h", ">","п", 1);
	add_keyboard_button_nongrid(this, FirstRow, "j", ",","р", 1);
	add_keyboard_button_nongrid(this, FirstRow, "k", "|","о", 1);
	add_keyboard_button_nongrid(this, FirstRow, "l", "?","л", 1);
	add_keyboard_button_nongrid(this, FirstRow, "m", "¢","д", 1); //Copyright

        if (shift==2){
	 add_keyboard_button_nongrid(this, FirstRow, "", "","ж", 1);
	 add_keyboard_button_nongrid(this, FirstRow, "", "","э", 1);	
	}
	grid->addLayout(FirstRow,2,0,1,10);


	FirstRow = new QHBoxLayout;
        FirstRow->setSpacing(3);
	FirstRow->setMargin(0);
	if(shift==1){
		add_keyboard_button(this, grid, "", COPY, 3, 0, 2);
		add_keyboard_button_nongrid(this, FirstRow, "", PASTE,"", 2);
	}else{
		add_keyboard_button(this, grid, CAPS, "", 3, 0, 2);
		add_keyboard_button_nongrid(this, FirstRow, "z", "","я", 1);
		add_keyboard_button_nongrid(this, FirstRow, "x", "","ч", 1);
	}
	
	add_keyboard_button_nongrid(this, FirstRow, "c", "/","с", 1);
	add_keyboard_button_nongrid(this, FirstRow, "v", ";","м", 1); //Pound
	add_keyboard_button_nongrid(this, FirstRow, "b", "\\","и", 1); //Yenn
	add_keyboard_button_nongrid(this, FirstRow, "n", "§","т", 1);
        if (shift==2){
	 add_keyboard_button_nongrid(this, FirstRow, "v", ";","ь", 1); //Pound
	 add_keyboard_button_nongrid(this, FirstRow, "b", "\\","б", 1); //Yenn
	 add_keyboard_button_nongrid(this, FirstRow, "n", "§","ю", 1);	
	}
	grid->addLayout(FirstRow, 3 ,2 , 1, 6);	
	add_keyboard_button(this, grid, BACKSPACE, BACKSPACE, 3, 8, 2);
        
	//add_keyboard_button(this, grid, SPACE, "", 4, 0, 8);
	//add_keyboard_button(this, grid, SHIFT, "", 4, 8, 2);
	add_keyboard_button(this, grid, SPACE, SPACE, 4, 0, 5);
	add_keyboard_button(this, grid, ",", ",", 4, 5, 1);
	add_keyboard_button(this, grid, ".", ".", 4, 6, 1);
	add_keyboard_button(this, grid, ENTER, ENTER, 4, 7, 3);
	this->setLayout(grid);
	
};



void MyKeyBoard::pressed(char *t){
	bool WorkMode = ((Dialog *)parent)->getSaveMode();	

	if(t == MyKeyBoard::BACKSPACE){
		if (!WorkMode){		
			QTextCursor c = text->textCursor();
			c.deletePreviousChar();		
		} else {
			filename->backspace();
		}
		cap_on = false;
	}else if(t == MyKeyBoard::SPACE){
		if (!WorkMode)
			text->insertPlainText(tr(" "));
		else 
			filename->insert(tr(""));
		cap_on = false;
	}else if(t == MyKeyBoard::CAPS){
		cap_on = true;
	}else if(t == MyKeyBoard::ENTER){
		if (!WorkMode) text->insertPlainText(tr("\n"));
	}else if(t == MyKeyBoard::COPY){
		if (!WorkMode)
			text->copy();
		else
			filename->copy();
	}else if(t == MyKeyBoard::PASTE){
		if (!WorkMode)
			text->paste();
		else
			filename->paste();
	}else{
		if (!WorkMode){		
			if(cap_on) text->insertPlainText(tr(t).toUpper());
			else text->insertPlainText(tr(t));
		} else {
			if(cap_on) filename->insert(tr(t).toUpper());
			else filename->insert(tr(t));	
		}
		cap_on = false;
	}
	if (!WorkMode) onyx::screen::ScreenProxy::instance().flush(text, onyx::screen::ScreenProxy::GU); else
			onyx::screen::ScreenProxy::instance().flush(filename, onyx::screen::ScreenProxy::GU);
}

MyKeyBoard::~MyKeyBoard(){

};

void MyKeyBoard::keyPressEvent(QKeyEvent* event){
	if (event->key() == Qt::Key_Escape){
		QApplication::exit();
	}else if (event->key() == Qt::Key_Menu){
		onyx::screen::ScreenProxy::instance().flush(parent, onyx::screen::ScreenProxy::GC);
	}
};

